import os
from trac.wiki.macros import WikiMacroBase
from StringIO import StringIO
from trac.config import PathOption, Option
from pysqlite2 import dbapi2
import cgi
import time
from trac.wiki.model import WikiPage

class ProjectsGroup(object):
	def __init__(self, label):
		self.__label = label
		self.__projects = []

	def label(self):
		return self.__label

	def add(self, project):
		self.__projects.append(project)

	def get_all(self):
		return self.__projects

class WstatsMacro(WikiMacroBase):
	'''
		This does not have nothing to do with having multiple projects in a trac but multiple tracs in a system. 
		In a multi-project environment with multiple tracs you'll probably want to know statistics about all trac 
		installed togheter, so that's what this plugin does, Getting ticket statistics and order them by last 
		time db modified.
		[[BR]]
		[[BR]]
		Configurations:
		{{{
		[wstats]
		env_parent_dir = /path/to/trac/environments
		trac_url = / # trac base url
		label_today = Today
		label_yesterday = Yesterday Tickets
		label_older = Older
		label_monthly = Monthly
		}}}
		[[BR]]
		[[BR]]
		Usage:
		{{{
		[[Wstats]]
		}}}
	'''
	
	env_parent_dir = PathOption("wstats", "env_parent_dir", "/srv/trac", "Trac environments parent dir")
	trac_url = PathOption("wstats", "trac_url", "/", "Trac URL")
	db_path = PathOption("wstats", "db_path", "/db/trac.db", "DB Path")
	label_today = Option("wstats", "label_today", "Today", "Label for 'Today' header")
	label_yesterday = Option("wstats", "label_yesterday", "Yesterday", "Label for 'Yesterday' header")
	label_older = Option("wstats", "label_older", "Older", "Label for 'Older' header")
	label_monthly = Option("wstats", "label_monthly", "Monthly", "Label for 'Monthly' header")
	label_nodata = Option("wstats", "label_nodata", "No Data", "Label for 'No Data' header")

	def expand_macro(self, formatter, name, args):
		f = cgi.FieldStorage()

		if f.has_key("show"):
			show = f["show"]
		else:
			show = None

		page = "/wiki/%s?show=" % WikiPage(self.env, formatter.context.resource).name 

		if show:  
			return self.print_header(show) + self.create_struct(show, 0, page + show)
		else:
			IO = StringIO()
			info = list()
			info.append(ProjectsGroup(self.label_today))
			info.append(ProjectsGroup(self.label_yesterday))
			info.append(ProjectsGroup(self.label_monthly))
			info.append(ProjectsGroup(self.label_older))

			for traca in os.listdir(self.env_parent_dir):
				project_base = self.env_parent_dir + os.path.sep + traca

				if not os.path.isdir(project_base):
					continue 

				mtime = os.stat(project_base + self.db_path).st_mtime # Mod date in seconds
				days = ((time.time() - mtime) / (1 * 86400))
				if days < 1: 
					info[0].add(traca)
				else: 
					if days < 2: 
						info[1].add(traca)
					else:
						if days < 30: 
							info[2].add(traca)
						else: 
							info[3].add(traca)
			
			for i in info:
				IO.write(self.print_header(i.label()))
				for tracn in i.get_all(): 
					a = self.create_struct(tracn , 0 , page + tracn)
					IO.write(a)
				IO.write (self.print_header(self.label_nodata, 2) if not i.get_all() else "")
				IO.write ("<hr/>")
			
			return self.print_header("Main") + "<p>Active tracs, ordered by databse modification time</p>" + IO.getvalue()

	def create_struct(self, trac, mode , page):
		url = self.trac_url + trac
	
		# Connect to database
		db_path = "%s/%s%s" % (self.env_parent_dir, trac, self.db_path)
		
		self.log.debug ("************************ Db is %s (%s)" % (db_path, self.env_parent_dir))
		try: 
			dbase = dbapi2.connect(db_path)
		except:	
			return "<h1> Could Not Open database </h1>"
		cur = dbase.cursor()
	
		# Get new, closed and total projects
		cur.execute('select count(id) from ticket where status!="closed";')
		new = cur.fetchone()
		new = int(new[0])
	
		cur.execute('select count(id) from ticket where status=="closed";')
		closed = cur.fetchone()
		closed = int(closed[0])
	
		cur.execute('select count(id) from ticket;')
		total = cur.fetchone()
		total = int(total[0])
		# Get lastest revision
		cur.execute("select * from revision limit 1 offset (select count(*) from revision) - 1 ")
		row = cur.fetchone()
	
		try:
			revision = "\n<br/>Lastest revision: <ul>\n <li>Author: %s</li>\n<li> Date:" % row[2]
			revision = revision + time.strftime("%d %b %y", time.localtime(row[1])) + "</li>\n"
			revision = revision + "<li> Message: %s </li>\n</ul>\n" % row[3]
		except: revision = ""
	
		style = " style=\"color:red;\"" if closed == total else "" 
	
		# Print results
		if mode == 0:
			try: closedpercentage = closed * 100 / (new + closed)
			except: closedpercentage = 0
			newpercentage = 100 - closedpercentage
			return "\n \
			<h2><a %s href=\"%s\"\>Trac %s</a></h2> \
			Tickets: %s <br/> \
		projectsickets: <a href=\"%s/query?status=assigned&status=needinfo&status=new&status=reopened&group=resolution&col=id&col=summary&col=status&col=type&col=priority&col=component&order=priority\"> %s (%s%%)</a> <br/>\
		projects tickets: <a href=\"%s/query?status=closed\"> %s (%s%%)</a> \
			%s\
			\n" % (style, url, trac, total, url, new, newpercentage, url, closed, closedpercentage, revision)
		else:
			return """
				<h2 >Trac: <a %s href="%s">%s</a> </h2>
				Total: %d<br/>
				Closed:%d <br/>
				New: %d<br/>
				<a href=%s>More Info</a><br/>%s<br/>
			""" % (style, url, trac, total , closed , new, page, revision)
	
	def print_header(self, trac, depth = 1): 
		return "<h%s> %s </h%s>" % (depth, trac, depth)
